//-------------- bFIFO  Byte Fifo------------------
struct bFIFO {
    unsigned char *buf;
    unsigned char *pin;
    unsigned char *pout;
    int    nshort;
};

struct bFIFO *bfifo_create(int nshort) {
    struct bFIFO *bfifo;

    bfifo = (struct bFIFO *)malloc(sizeof(struct bFIFO));
    assert(bfifo != NULL);

    bfifo->buf = (unsigned char*)malloc(sizeof(char)*nshort);
    assert(bfifo->buf != NULL);
    bfifo->pin = bfifo->buf;
    bfifo->pout = bfifo->buf;
    bfifo->nshort = nshort;

    return bfifo;
}

void bfifo_destroy(struct bFIFO *cfifo) {
    assert(bfifo != NULL);
    free(cfifo->buf);
    free(cfifo);
}

int bfifo_used(const struct bFIFO * const bfifo)
{
    unsigned char         *pin = bfifo->pin;
    unsigned char         *pout = bfifo->pout;
    unsigned int   used;

    assert(bfifo != NULL);
    if (pin >= pout)
        used = pin - pout;
    else
        used = bfifo->nshort + (unsigned int)(pin - pout);

    return used;
}

int bfifo_free(const struct bFIFO * const bfifo)
{
    // available storage is one less than nshort as prd == pwr
    // is reserved for empty rather than full

    return bfifo->nshort - bfifo_used(bfifo) - 1;
}

int bfifo_write(struct bFIFO *bfifo, unsigned char data[], int n) {
    int          	  i;
    unsigned char         *pdata;
    unsigned char         *pin = bfifo->pin;

    assert(bfifo != NULL);
    assert(data != NULL);

    if (n > bfifo_free(bfifo)) {
	return -1;
    }
    else {

	/* This could be made more efficient with block copies
	   using memcpy */

	pdata = data;
	for(i=0; i<n; i++) {
	    *pin++ = *pdata++;
	    if (pin == (bfifo->buf + bfifo->nshort))
		pin = bfifo->buf;
	}
	bfifo->pin = pin;
    }

    return 0;
}

int bfifo_read(struct bFIFO *bfifo, unsigned char data[], int n)
{
    int            i;
    unsigned char         *pdata;
    unsigned char         *pout = bfifo->pout;

    assert(bfifo != NULL);
    assert(data != NULL);

    if (n > bfifo_used(bfifo)) {
	return -1;
    }
    else {

	/* This could be made more efficient with block copies
	   using memcpy */

	pdata = data;
	for(i=0; i<n; i++) {
	    *pdata++ = *pout++;
	    if (pout == (bfifo->buf + bfifo->nshort))
		pout = bfifo->buf;
	}
	bfifo->pout = pout;
    }

    return 0;
}
//-------------- sFIFO  Short Fifo------------------
struct sFIFO {
    short *buf;
    short *pin;
    short *pout;
    int    nshort;
};

struct sFIFO *sfifo_create(int nshort) {
    struct sFIFO *sfifo;

    sfifo = (struct sFIFO *)malloc(sizeof(struct sFIFO));
    assert(bfifo != NULL);

    sfifo->buf = (short*)malloc(sizeof(short)*nshort);
    assert(bfifo->buf != NULL);
    sfifo->pin = sfifo->buf;
    sfifo->pout = sfifo->buf;
    sfifo->nshort = nshort;

    return sfifo;
}

void sfifo_destroy(struct sFIFO *cfifo) {
    assert(sfifo != NULL);
    free(cfifo->buf);
    free(cfifo);
}

int sfifo_used(const struct sFIFO * const sfifo)
{
    short         *pin = sfifo->pin;
    short         *pout = sfifo->pout;
    int   used;

    assert(sfifo != NULL);
    if (pin >= pout)
        used = (int)(pin - pout);
    else
        used = sfifo->nshort + (int)(pin - pout);

    return used;
}

int sfifo_free(const struct sFIFO * const sfifo)
{
    // available storage is one less than nshort as prd == pwr
    // is reserved for empty rather than full

    return sfifo->nshort - sfifo_used(sfifo) - 1;
}

int sfifo_write(struct sFIFO *sfifo, short data[], int n) {
    int          	  i;
    short         *pdata;
    short         *pin = sfifo->pin;

    assert(sfifo != NULL);
    assert(data != NULL);

    if (n > sfifo_free(sfifo)) {
	return -1;
    }
    else {

	/* This could be made more efficient with block copies
	   using memcpy */

	pdata = data;
	for(i=0; i<n; i++) {
	    *pin++ = *pdata++;
	    if (pin == (sfifo->buf + sfifo->nshort))
		pin = sfifo->buf;
	}
	sfifo->pin = pin;
    }

    return 0;
}

int sfifo_read(struct sFIFO *sfifo, short data[], int n)
{
    int            i;
    short         *pdata;
    short         *pout = sfifo->pout;

    assert(sfifo != NULL);
    assert(data != NULL);

    if (n > sfifo_used(sfifo)) {
	return -1;
    }
    else {

	/* This could be made more efficient with block copies
	   using memcpy */

	pdata = data;
	for(i=0; i<n; i++) {
	    *pdata++ = *pout++;
	    if (pout == (sfifo->buf + sfifo->nshort))
		pout = sfifo->buf;
	}
	sfifo->pout = pout;
    }

    return 0;
}

